package net.rumati.logging.muffero.util;

import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class ReadWriteLockedMap<K, V>
    implements Map<K, V>
{
    private final Map<K, V> map;
    private final ReadWriteLock lock;

    public ReadWriteLockedMap(Map<K, V> map)
    {
        this(map, new ReentrantReadWriteLock(true));
    }

    public ReadWriteLockedMap(Map<K, V> map, ReadWriteLock lock)
    {
        this.map = map;
        this.lock = lock;
    }

    public int size()
    {
        lock.readLock().lock();
        try {
            return map.size();
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean isEmpty()
    {
        lock.readLock().lock();
        try {
            return map.isEmpty();
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean containsKey(Object key)
    {
        lock.readLock().lock();
        try {
            return map.containsKey(key);
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean containsValue(Object value)
    {
        lock.readLock().lock();
        try {
            return map.containsValue(value);
        }finally{
            lock.readLock().unlock();
        }
    }

    public V get(Object key)
    {
        lock.readLock().lock();
        try {
            return map.get(key);
        }finally{
            lock.readLock().unlock();
        }
    }

    public V put(K key, V value)
    {
        lock.writeLock().lock();
        try {
            return map.put(key, value);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public V remove(Object key)
    {
        lock.writeLock().lock();
        try {
            return map.remove(key);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public void putAll(Map<? extends K, ? extends V> m)
    {
        lock.writeLock().lock();
        try {
            map.putAll(m);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public void clear()
    {
        lock.writeLock().lock();
        try {
            map.clear();
        }finally{
            lock.writeLock().unlock();
        }
    }

    public Set<K> keySet()
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedSet<K>(map.keySet(), lock);
        }finally{
            lock.readLock().unlock();
        }
    }

    public Collection<V> values()
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedCollection<V>(map.values(), lock);
        }finally{
            lock.readLock().unlock();
        }
    }

    public Set<Entry<K, V>> entrySet()
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedSet<Entry<K, V>>(map.entrySet(), lock);
        }finally{
            lock.readLock().unlock();
        }
    }
    
    public void readLock()
    {
        lock.readLock().lock();
    }

    public void readUnlock()
    {
        lock.readLock().unlock();
    }
}
